import 'package:json_annotation/json_annotation.dart';
import 'package:lori_sdk/data/models/product/product.dart';

part 'skuAttribute.g.dart';

//Sku Attribute object returned from the API
@JsonSerializable()
class SkuAttribute {
  SkuAttribute();

  String id;
  String skuId;
  SkuAttributeType type;
  String value;
  String label;
  String order;

  factory SkuAttribute.fromJson(Map<String, dynamic> json) => _$SkuAttributeFromJson(json);

  Map<String, dynamic> toJson() => _$SkuAttributeToJson(this);
}

//Sku Attribute available type
enum SkuAttributeType {
  @JsonValue(0)
  text,

  @JsonValue(1)
  color,

  @JsonValue(2)
  picker
}

//Type created to simplify the UI construction
class SkuAttributeValues {
  String label;
  SkuAttributeType type;
  String order;
  List<String> values;
}

extension SkuAttributes on List<Sku> {
  List<SkuAttributeValues> get availableAttributes {
    //Get the label of the attributes, ordered by the attribute order
    var firstSkuAttrs = this.first.attributes;
    firstSkuAttrs.sort((la, ra) => la.order.compareTo(ra.order));
    var attrLabels = firstSkuAttrs.map((a) => a.label).toList();

    this.forEach((sku) {
      var skuAttrs = sku.attributes;
      skuAttrs.sort((la, ra) => la.order.compareTo(ra.order));
      var skuAttrLabels = skuAttrs.map((a) => a.label).toList();

      if (attrLabels != skuAttrLabels) {
        //If any sku has a different sku attributes than the others we can't use the sku attributes
        return null;
      }
    });

    //Get ALL attributes and values available
    var allAttributes = this.expand((sku) => sku.attributes);

    //Get all labels available
    var allLabels = allAttributes.map((attr) => attr.label).toSet().toList();

    //Create the array to hold the available attributes
    List<SkuAttributeValues> availableAttributes = List<SkuAttributeValues>();

    //Loop through all the labels
    allLabels.forEach((label) {
      //Get all atrributes with this label
      var attributesWithLabel = allAttributes.where((a) => a.label == label);

      //Create the wrapper
      var attributeValues = SkuAttributeValues();
      attributeValues.label = label;
      //The type should be equal on all attributes with the same label, so pick the first
      attributeValues.type = attributesWithLabel.first.type;
      //The same is valid to order
      attributeValues.order = attributesWithLabel.first.order;
      //Create an array with all the values
      attributeValues.values = attributesWithLabel.map((a) => a.value).toList();
      //Sort the values
      attributeValues.values.sort((lv, rv) => lv.compareTo(rv));

      availableAttributes.add(attributeValues);
    });

    return availableAttributes;
  }
}